% advec_diff_analysis.m
% Bernd Fischer: 06.09.96
%
% -- calculates eigenvalues of either Galerkin or 
%    SUPG matrix for wind (0,1)
%
% -- assembles the SUPG matrix for arbitrary wind
%
% -- applies GMRES to the generated matrix 
%
% change epsilon, Delta, n as required
%
% uses:  eigFsup_Alison, rectangle_ellipse, generate_Fsup, fgmres

  clear

%%Fsup parameter
   n=16;            %Gridsize:  h=1/n;
   theta=0.0;          %wind direction: beta_x=-sin(theta); beta_y=cos(theta); 
   epsilon=0.01;      %viscosity parameter
   Delta=[.6];   %SUPG parameter(s); Galerkin case:Delta=0

   
%%GMRES parameter  
   N=(n-1)*(n-1); rand('seed',0);  
   
   b    =zeros(N,1); %right hand side
   x0   =rand(N,1);  %initial guess
   itmax=51;         %upper bound on number of iterations
   tol  =1e-13;      %tolerance for stopping criterion |rk|/b <= tol            
   
%%%%%%%%%%

   h=1/n; nx=n-1; ny=nx; 
   beta_x=-sin(theta); beta_y=cos(theta); 
   Colour=['w','c','m','b','r','g'];
   
   K1='eigenvalues and ellipses';
   K2='eigenvalues, ellipses and GMRES';
   K3='only GMRES';
   
   K=menu('think positive',K1,K2,K3);
    
   for l=1:length(Delta)
      delta=Delta(l);
      colour=Colour(rem(l,length(Colour)));
   
      if K <=2
   %%compute eigenvalues of Fsup by Alisons formulae
      [eigFsup,lambda,mu,gamma]=...
          eigFsup_Alison(h,nx,epsilon,delta,beta_x,beta_y);
   %%plot eigenvalues
      figure(1)   
      plot(real(eigFsup),imag(eigFsup),[colour,'.'])
      if l== 1, hold on, end

   %%compute rectangle (corners are given by minRe,maxRe,minIm,maxIm)
   %%which contains the spectrum of Fsup,
   %%an ellipse (midpoint d_ell, semi axis a_ell and b_ell,
   %%foci d_ell +- c_ell) of smallest area enclosing this rectangle,
   %%and the corresponding asymptotic convergence factor r/R

      [minRe,maxRe,minIm,maxIm,d_ell,a_ell,b_ell,r,R]=...
          rectangle_ellipse(h,epsilon,delta);
		  
      asymp(l)=r/R;

      if max(abs(imag(eigFsup))) > 0 
         t=linspace(0,2*pi,100);
         plot(d_ell+a_ell*cos(t)+sqrt(-1)*b_ell*sin(t),colour);
   
         plot([minRe, maxRe],[minIm, minIm],colour); 
         plot([minRe, maxRe],[maxIm, maxIm],colour);
         plot([minRe, minRe],[minIm, maxIm],colour); 
         plot([maxRe, maxRe],[minIm, maxIm],colour);
   
      end
   
      if l==length(Delta)
		 title(['n = ', int2str(n),', epsilon = ', num2str(epsilon),...
		        ', beta_x = ',num2str(beta_x), ', beta_y = ',num2str(beta_y)])
				deltastr=''; asympstr='';
		 for j=1:length(Delta)-1
		     deltastr=[deltastr,num2str(Delta(j)),'(',Colour(j),')',', '];
			 asympstr=[asympstr,num2str(asymp(j)),'(',Colour(j),')',', '];
		 end
		 deltastr=[deltastr,num2str(Delta(length(Delta))),'(',Colour(length(Delta)),')'];
		 asympstr=[asympstr,num2str(asymp(length(Delta))),'(',Colour(length(Delta)),')'];
		 xlabel(['delta = ', deltastr,',   asymp_factor = ',asympstr])
         axis('square')
         hold off
      end
      end
   
      if K>=2
   %%generate Fsup
         [Fsup]=generate_Fsup(h,nx,epsilon,delta,beta_x,beta_y);
		 
   %%apply GMRES to Fsup (without preconditioning)	    
         [x,plerr] = fgmres(Fsup,b,x0,itmax,tol,'pmethod');
	  
	     figure(2)
         semilogy(0:length(plerr)-1,plerr,colour);
	     if l==1, hold on, end 
	     if l==length(Delta)
            title(['n = ', int2str(n),', epsilon = ', num2str(epsilon),...
		        ', beta_x = ',num2str(beta_x), ', beta_y = ',num2str(beta_y)])
			deltastr='';
		    for j=1:length(Delta)-1
		        deltastr=[deltastr,num2str(Delta(j)),'(',Colour(j),')',', '];
		    end
		    deltastr=[deltastr,num2str(Delta(length(Delta))),'(',Colour(length(Delta)),')'];
		    xlabel(['delta = ', deltastr])
            ylabel('GMRES residual error')
            axis('square')
		    hold off
	     end
      end
   end
return
