    function [x, plotres] = ...
         bcglong(A, rhs, x0, n_max, tol, matvec)
%   function [x, plotres] = ...
%        bcglong(A, rhs, x0, n_max, tol, matvec)
%   BF: 04.01.97
%   computes the solution of Ax = rhs using the  scheme.
% 
%   startvector:                      x0
%   maximal number of iterations:     n_max
%   desired accuracy of the residual: tol
%
%   uses: matvec
%

    %   initialize
    n            =   1;
    N            =   length(x0);
    x            =   x0;
    r            =   rhs - feval( matvec, A, x );
	s            =   r;
    sr           =   s' * r;
    sr_old       =   1;
    norm_r       =   norm(r);
    norm_r0      =   norm_r;
	
    w            =   zeros(N, 1);
	w_star       =   w;

    plotres      =   zeros(n_max, 1);
    plotres(n)   =   (norm_r / norm_r0);
	
	fprintf('\n    itn    res_BCG  \n');
    

    %   iterate
    while (n < n_max + 1) &  (norm_r / norm_r0 > tol),

        n        =   n+1;
        nu       =   sr / sr_old;
        w        =   nu * w - r;
        Aw       =   feval(matvec, A, w);
		w_star   =   nu * w_star - s;
		Atw_star =   feval(matvec, A', w_star);
        eta      =   - sr / (w_star' * Aw);

        %   update
        x         =   x + eta * w;
        r         =   r - eta * Aw;
		s         =   s - eta * Atw_star;

        sr_old  =   sr;
        sr      =   s' * r;
        
		
		norm_r=norm(rhs - feval( matvec, A, x ));
        plotres(n)=   ( norm_r / norm_r0 );

         
		fprintf('  %5i     %8.4f \n',n-1,log10(plotres(n)));
		
    end;    %   while

    plotres       =   plotres(1:n);

return; 
