function [x,plerr] = bicgstab(A,b,x0,itmax,tol,pmethod)
%[x,plerr] = bicgstab(A,b,x0,itmax,tol,pmethod)
% Version 07.12.96 by Bernd Fischer
%
% BICGSTAB algorithm for solving linear systems Ax=b, 
% with right-oriented preconditioning.  
% Arguments are as follows:
%   A                coefficient matrix
%   b                right hand side
%   x0               initial guess
%   itmax            upper bound on number of iterations
%   tol              tolerance for stopping criterion |rk|/b <= tol
%   pmethod          preconditioning method
%
% uses: matvec and Msolve 

%%Initialize
   r = b-matvec(A,x0); r_hat=r;
   p=r; x=x0; rho=r'*r_hat;
   
   norm_r=norm(r); norm_r0=norm_r;
   j=1; plerr(j)=norm_r/norm_r0;
   fprintf('\n    itn    res_BICGSTAB   \n');
   
%%Iterate
   while (norm_r > tol*norm_r0) & (j < itmax)
      j=j+1;
	  
	  Ap=A*p;
	  alpha=rho/(Ap'*r_hat);
	  s=r-alpha*Ap;
	  As=A*s;
	  w=As'*s/(As'*As);
	  
	  x=x+alpha*p+w*s;
	  r=s-w*As;
	  
	  rho_old=rho; rho=r'*r_hat;
	  beta=(rho/rho_old)*(alpha/w);
	  p=r+beta*(p-w*Ap);
	  
	  norm_r=norm(b-matvec(A,x));
	  plerr(j)=norm_r/norm_r0;
	  fprintf('  %5i     %8.4f \n',j-1,log10(plerr(j)));
   end;
   
return
