function [eigFsup,lambda,mu,gamma]=...
          eigFsup_Alison(h,nx,epsilon,delta,beta_x,beta_y)
%
% computes the eigenvalues eigFsup of the matrix Fsup 
% and the eigenvalues lambda,mu,gamma of the corresponding submatrices 
% based upon Alisons analysis
%
% ??eigenvectors are still to be implemented

%%matrix entries 
   ca=8/3*epsilon+delta*h*4/3*(beta_x*beta_x+beta_y*beta_y);
   cb=-1/3*epsilon+h/3*beta_x+delta*h/3*(-2*beta_x*beta_x+beta_y*beta_y); 
   cc=-1/3*epsilon-h/3*beta_x+delta*h/3*(-2*beta_x*beta_x+beta_y*beta_y); 
   cd=-1/3*epsilon+h/3*beta_y+delta*h/3*(beta_x*beta_x-2*beta_y*beta_y); 
   cg=-1/3*epsilon-h/3*beta_y+delta*h/3*(beta_x*beta_x-2*beta_y*beta_y); 
   ce=-1/3*epsilon+h/12*(beta_x+beta_y)-...
       delta*h*(1/6*(beta_x*beta_x+beta_y*beta_y)+1/2*beta_x*beta_y); 
   cf=-1/3*epsilon+h/12*(-beta_x+beta_y)-...
       delta*h*(1/6*(beta_x*beta_x+beta_y*beta_y)-1/2*beta_x*beta_y); 
   ch=-1/3*epsilon+h/12*(beta_x-beta_y)-...
       delta*h*(1/6*(beta_x*beta_x+beta_y*beta_y)-1/2*beta_x*beta_y); 
   ci=-1/3*epsilon-h/12*(beta_x+beta_y)-...
       delta*h*(1/6*(beta_x*beta_x+beta_y*beta_y)+1/2*beta_x*beta_y); 
   
%%compute eigenvalues of Fsup by Alisons formulae
   j=1:nx; n=nx+1;
   cos_j=cos(pi*j/n);
   gamma =cg+2.*sign(ch)*sqrt(ci*ch)*cos_j ;  %eigenvalues of left block
   lambda=ca+2.*sign(cb)*sqrt(cc*cb)*cos_j ;  %eigenvalues of middle block
   mu    =cd+2.*sign(ce)*sqrt(cf*ce)*cos_j ;  %eigenvalues of right block
   
   mat_eigFsup=(2*sign(mu).*sqrt(gamma.*mu))'*cos_j;  
   for k=1:nx mat_eigFsup(1:nx,k)=mat_eigFsup(1:nx,k)+lambda'; end 
   
   eigFsup=reshape(mat_eigFsup,nx*nx,1);   %eigenvalues of Fsup
   
return
