function [x,plerr] = fgmres(A,b,x0,itmax,tol,pmethod);
%
% [xgmr,rgmr] = fgmres(A,b,x0,itmax,tol,pmethod);
% classic GMRES algorithm for solving linear systems Ax=b, 
% with right-oriented preconditioning.  Arguments are as follows:
%   A                coefficient matrix
%   b                right hand side
%   x0               initial guess
%   itmax            upper bound on number of iterations
%   tol              tolerance for stopping criterion |rk|/b <= tol
%   pmethod          preconditioning method
%
% calls external functions: matvec and Msolve 
%       David Silvester 
%       November 11, 1994
m=itmax;              % restart parameter
M=speye(size(A));
%%%% r = b - feval(matmult,M1,M2,M3,'A',x0);
r = b-matvec(A,x0);
bn = norm(r);
j = 1;            
plerr(j)=1.0;
% fprintf('  %5i     %8.4f \n',j,log10(norm(r)));
   fprintf('\n')
   disp(['GMRES ',setstr(pmethod),' preconditioning']); 
   fprintf('\n    itn    res_GMRES   \n');
n=length(x0);

% Step 1: Start with initial residual
% v=zeros(n,m); hm=zeros(m+1,m);
beta=bn; v(:,1)=r/beta;  

while norm(r) > tol*bn & j < itmax,
% Step 2: Arnoldi iteration, modified Gram-Schmidt form
%%%%%   q = feval(qinv,Q1,Q2,Q3,Q4,Q5,qopts,'Q',v(:,j));
    q = Msolve(M,v(:,j));
%%%%   vhat = feval(matmult,M1,M2,M3,'A',q);
   vhat = matvec(A,q);
   for i=1:j,
      hm(i,j) = vhat'*v(:,i);
      vhat    = vhat - hm(i,j)*v(:,i);
   end 
   hm(j+1,j) = norm(vhat);

   % Step 3: Form the approximate solution and residual
   y = hm \ [beta,zeros(1,j)]';
%%%%   x = x0 + feval(qinv,Q1,Q2,Q3,Q4,Q5,qopts,'Q',v*y);
   x = x0 + Msolve(M,v*y);
%%%%   r = b - feval(matmult,M1,M2,M3,'A',x);
   r = b-matvec(A,x);
   %plerr(j+1)=norm(r)/bn;
   plerr(j+1)=norm(r);

   if j<m, v(:,j+1) = vhat/hm(j+1,j); end
   j=j+1;
   fprintf('  %5i     %8.4f \n',j-1,log10(norm(r)));

end %while

% plot convergence history
 %semilogy(0:j-1,plerr);
 %title('GMRES Convergence History')
 %xlabel('iteration number')
 %ylabel('residual error')
 %axis('square')
% axis([0 25 1e-6 1.0])

return;
