% plot_asymp_true.m
% Bernd Fischer: 19.09.96
%
% computes the asymptotic convergence rate w.r.t. delta
% for fixed h and epsilon 
%
% important cases:  delta_U wins: n=16, epsilon=0.001
%                   delta_L wins: n=16, epsilon=0.01
%
% uses: eigFsup_Alison

%%Fsup parameter
   n=16;                     %Gridsize:  h=1/n; 
   epsilon=0.001;             %viscosity parameter
   dis=200;                  %number of sample points
   Delta=linspace(0,1,dis);  %SUPG parameter(s); Galerkin case:Delta=0
   
%%critical parameter
   delta_L=1/2-epsilon/h;   %mu*gamma(1)=0: "spitting starts"
   delta_U=1/2+epsilon/h;   %mu*gamma(-1)=0:all eigenvalues are real
   delta_S=+epsilon/h;      %square
   
   Delta=[Delta,delta_L,delta_U];

%%%%%%%%%

   h=1/n;
   theta=0.0;                            %wind direction 
   beta_x=-sin(theta); beta_y=cos(theta); 
   
%%compute asymptotic convergence factor conv
   for j=1:length(Delta)
      [eigFsup,lambda,mu,gamma]=...
          eigFsup_Alison(h,n-1,epsilon,Delta(j),beta_x,beta_y);
	  minRe=min(real(eigFsup)); maxRe=max(real(eigFsup));
	  maxIm=max(imag(eigFsup));
	  D(j)=(minRe+maxRe)/2; A(j)=(maxRe-minRe)/2; B(j)=maxIm;
	  
	  if B(j)==0
	     conv(j)=A(j)/(D(j)+sqrt(D(j)^2-A(j)^2));
	   else
	     conv(j)=sqrt(2)*(A(j)+B(j))/(D(j)+sqrt(D(j)^2+2*abs(B(j)^2-A(j)^2)));
		 if D(j) < sqrt(2)*A(j), conv(j)=1; end
	  end
   end
   
   I1=min(find(B==0)); I2=min(find(conv==1));
   conv_L=conv(dis+1);
   conv_U=conv(dis+2);
   
   
%%plot asymptotic convergence rates
%%note: it is assumed that I2<I1
   figure(1)
   if length(I2) == 0
      plot(Delta(1:I1-1),conv(1:I1-1));
      hold on
      plot(Delta(I1:dis),conv(I1:dis));
    else
	  plot(Delta(1:I2-1),conv(1:I2-1));
      hold on
      plot(Delta(I1:dis),conv(I1:dis));
   end
   plot(delta_L,conv_L,'o')
   plot(delta_U,conv_U,'o')
   title(['n = ', int2str(n),', epsilon = ', num2str(epsilon),...
		        ', beta_x = ',num2str(beta_x), ', beta_y = ',num2str(beta_y)])
   xlabel(['delta_L = ', num2str(delta_L),',  ','delta_U = ', num2str(delta_U)])
   hold off
   
   M=max([D,A,B]);
   figure(2)
   plot(Delta(1:dis),D(1:dis))
   hold on
   plot(Delta(1:dis),A(1:dis))
   plot(Delta(1:dis),B(1:dis))
   plot([delta_L,delta_L],[0,M])
   plot([delta_U,delta_U],[0,M])
   plot([delta_S,delta_S],[0,M])
   hold off
   

return
		  
