% plot_asympconv.m
% Bernd Fischer: 06.09.96
%
% computes the asymptotic convergence rate w.r.t. delta
% for fixed h and epsilon 
%
% important cases:  delta_U wins: n=32, epsilon=0.001
%                   delta_L wins: n=16, epsilon=0.01
%
% uses: rectangle_ellipse

%%Fsup parameter
   n=32;                     %Gridsize:  h=1/n; 
   epsilon=0.001;             %viscosity parameter
   Delta=linspace(0,2,200);  %SUPG parameter(s); Galerkin case:Delta=0

%%%%%%%%%

   h=1/n;
   theta=0.0;                            %wind direction 
   beta_x=-sin(theta); beta_y=cos(theta); 
   
%%compute asymptotic convergence factor r/R
   for j=1:length(Delta)
      [minRe,maxRe,minIm,maxIm,d_ell,a_ell,b_ell,r,R]=...
          rectangle_ellipse(h,epsilon,Delta(j));
      conv(j)=r/R;
	  %D=(minRe+maxRe)/2; A=(maxRe-minRe)/2; B=maxIm;
	  %conv1(j)=sqrt(2)*(A+B)/(D+sqrt(D^2+2*abs(B^2-A^2)));
   end

%%critical parameter
   delta_L=1/2-epsilon/h;   %mu*gamma(1)=0: "spitting starts"
   delta_U=1/2+epsilon/h;   %mu*gamma(-1)=0:all eigenvalues are real

   [minRe,maxRe,minIm,maxIm,d_ell,a_ell,b_ell,r,R]=...
          rectangle_ellipse(h,epsilon,delta_L);
   conv_L=r/R;
   [minRe,maxRe,minIm,maxIm,d_ell,a_ell,b_ell,r,R]=...
          rectangle_ellipse(h,epsilon,delta_U);
   conv_U=r/R;
   
   
%%plot asymptotic convergence rates
   plot(Delta,conv);
   hold on
   plot(delta_L,conv_L,'o')
   plot(delta_U,conv_U,'o')
   title(['n = ', int2str(n),', epsilon = ', num2str(epsilon),...
		        ', beta_x = ',num2str(beta_x), ', beta_y = ',num2str(beta_y)])
   xlabel(['delta_L = ', num2str(delta_L),',  ','delta_U = ', num2str(delta_U)])
   hold off

return
		  
