function [minRe,maxRe,minIm,maxIm,d_ell,a_ell,b_ell,r,R]=...
          rectangle_ellipse(h,epsilon,delta)
% Bernd Fischer: 06.09.96
%
% computes rectangle (corners are given by minRe,maxRe,minIm,maxIm)
% which contains the spectrum of Fsup,
% an ellipse (midpoint d_ell, semi axis a_ell and b_ell,
% foci d_ell +- c_ell) of smallest area enclosing this rectangle,
% and the corresponding asymptotic convergence factor r/R
%

%%compute rectangle which encloses the spectrum 
%%extremum x of p(t)=mu(t)*gamma(t); t=cos   
   x =-2*(h^2-2*delta*h*epsilon-4*(delta*h+epsilon)^2)/...
        (h^2-4*(delta*h+2*epsilon)^2);
   px =h^2*epsilon^2/(h^2-4*(delta*h+2*epsilon)^2);  %p(x)
   
   p1 =-(h^2-4*(delta*h+epsilon)^2)/4;               %p(1)
   pm1=-(h^2-4*(delta*h-epsilon)^2)/36;              %p(-1)

%%extrem values of lambda(t); t=cos
   la1 =2*(epsilon+delta*h);                         %lambda(1)
   lam1=2*(5*epsilon+delta*h)/3;                     %lambda(-1)

  %case 1
  %mu*gamma(t) < 0  for t \in [-1,1], x \not \in [-1,1]
   if p1 <= 0 & pm1 <=0 & abs(x) >1
      pmax=max([-p1,-pm1]); 
	  minRe=min([la1,lam1]); maxRe=max([la1,lam1]); 
	  minIm=-2*sqrt(pmax); maxIm=-minIm;
   end
   
  %case 2
  %mu*gamma(t) < 0  for t \in [-1,1], x \in [-1,1]
   if p1 <= 0 & pm1 <=0 & abs(x) <=1
      pmax=-px; 
	  minRe=min([la1,lam1]); maxRe=max([la1,lam1]); 
	  minIm=-2*sqrt(pmax); maxIm=-minIm;
   end
   
   %case 3
   %mu*gamma(1) > 0  and mu*gamma(-1) < 0, x \in [-1,1]
   if p1 > 0 & pm1 <=0 & abs(x) <= 1
      pmax=-px; 
	  minRe=min([la1-2*sqrt(p1),lam1]); maxRe=max([la1+2*sqrt(p1),lam1]); 
	  minIm=-2*sqrt(pmax); maxIm=-minIm;
   end
   
   %case 4
   %mu*gamma(1) > 0  and mu*gamma(-1) < 0, x \not \in [-1,1]
   if p1 > 0 & pm1 <=0  & abs(x) > 1
      pmax=-pm1; 
	  minRe=min([la1-2*sqrt(p1),lam1]); maxRe=max([la1+2*sqrt(p1),lam1]); 
	  minIm=-2*sqrt(pmax); maxIm=-minIm;
   end
   
   %case 5
   %mu*gamma(t) > 0  for t \in [-1,1] 
   if p1 > 0 & pm1 > 0 
	  minRe=min([la1-2*sqrt(p1),lam1-2*sqrt(pm1)]); 
	  maxRe=la1+2*sqrt(p1); 
	  minIm=0; maxIm=-minIm;
   end
   
   if minIm==0
      %%compute convergence factor for the intervall 
	  %%[minRe, maxRe]=[d_int-b_int,d_int+b_int]
        d_int=(maxRe+minRe)/2; b_int=maxRe-d_int;
		R=(d_int+sqrt(d_int^2-b_int^2))/b_int;
		r=1;
	else	  
      %%compute ellipse of smallest area enclosing
      %%the eigenvalue rectangle (cf. Saad (Book), pp. 254)
      %%midpoint d_ell
      %%semi axis a_ell and b_ell
      %%foci d_ell +- c_ell
        x1=minRe; x2=maxRe; y1=maxIm;
        B=(x1+x2)/2; A=(x2-x1)/2; T=y1;
        c_ell=sqrt(2*(A^2-T^2)); 
        a_ell=sqrt(2)*A;
        b_ell=sqrt(a_ell^2-c_ell^2);  
        d_ell=B; 
   
     %%compute convergence parameter (cf. Freund (Habil) pp. 36)
     %%F (Dis, pp.25)
       R=(d_ell+sqrt(d_ell^2+abs(c_ell)^2))/abs(c_ell);
       r=(abs(a_ell)+abs(b_ell))/abs(c_ell);
   end
   
return;
   
