% test.m
% Bernd Fischer: 07.12.96
%
%  SUPG matrix for wind (0,1)
%
% -- assembles the SUPG matrix for arbitrary wind
%
% -- applies various solvers to the generated matrix 
%
% change epsilon, Delta, n as required
%
% uses:   generate_Fsup, fgmres

  clear

%%Fsup parameter
   n=26;            %Gridsize:  h=1/n;
   theta=0.0;          %wind direction: beta_x=-sin(theta); beta_y=cos(theta); 
   epsilon=0.01;      %viscosity parameter
   delta=.6;   %SUPG parameter; Galerkin case:delta=0

   
%%solver parameter  
   N=(n-1)*(n-1); rand('seed',0);  
   
   b    =zeros(N,1); %right hand side
   x0   =rand(N,1);  %initial guess
   itmax=100;         %upper bound on number of iterations
   restart=20;       %restart value
   tol  =1e-13;      %tolerance for stopping criterion |rk|/b <= tol            
   

%%generate Fsup
   h=1/n; nx=n-1; ny=nx; 
   beta_x=-sin(theta); beta_y=cos(theta); 
   [Fsup]=generate_Fsup(h,nx,epsilon,delta,beta_x,beta_y);
		 
%%apply GMRES to Fsup (without preconditioning)	
   xm=x0;
   [x,GMRES_m] = fgmres(Fsup,b,xm,restart,tol,'pmethod');
   xm=x; GMRES_restart=GMRES_m;
   for j=1:1 
       [x,GMRES_m] = fgmres(Fsup,b,xm,restart,tol,'pmethod');
	   xm=x; GMRES_restart=[GMRES_restart,GMRES_m(2:length(GMRES_m))];
   end
   norm_r0 = norm(b-matvec(Fsup,x0));
   GMRES_restart=GMRES_restart/norm_r0;

    
   [x,GMRES] = fgmres(Fsup,b,x0,itmax,tol,'pmethod');
   GMRES=GMRES/norm_r0;
   [x,BICGSTAB] = bicgstab(Fsup,b,x0,itmax,tol,'pmethod');
   [x, BCG] = bcglong(Fsup, b, x0, itmax, tol, 'matvec');
   
   [x,res_norm,iter,flag,CGNR]=cgnr(Fsup,b,tol,itmax,x0);
   
   M1=eye(size(Fsup)); M2=M1;
   [x, error, iter, flag, qmr] = qmr( Fsup, x0, b, M1,M2, itmax, tol );
	  
%%plot convergence history	    
   semilogy(0:length(GMRES)-1,GMRES,'--');
   hold on
   semilogy(0:2:2*length(BICGSTAB)-1,BICGSTAB);
   hold off
	     
		    
return
